
#include <stdlib.h>
#include <string.h>

#define NO_JVMDI_MACROS
#include <jvmdi.h>
#include <jni.h>
#include "err.h"

extern JVMDI_Interface_1 *jvmdi;

/*
 * The resulting string should be free()d when the calling code is finished
 * with it.
 */
char *make_class_key(JNIEnv *env, jclass clazz, const char *suffix)
{
	jvmdiError err;
	char *clazz_sig = NULL;
	char *key_cursor;
	char *result;


	err = jvmdi->GetClassSignature(clazz, &clazz_sig);
	if(err != JVMDI_ERROR_NONE) {
		report_jvmdi_error("Can't get class details", err);
		return NULL;
	}

	result = malloc(strlen(clazz_sig) + strlen(suffix) + 1);

	/*
	 * REVISIT: jvmdicover can include the hashCode of the classloder in
	 *          the key name.  Here's the place to add similar code to this
	 *          software.
	 */

	if (result) {
		key_cursor = result;
		strcpy(key_cursor, clazz_sig);
		key_cursor += strlen(clazz_sig);
		strcpy(key_cursor, suffix);
	} else {
		EPRINT("make_class_key() failed to allocate mem for result");
	}

	jvmdi->Deallocate((jbyte *)clazz_sig);


	return result;
}


/*
 * The resulting string should be free()d when the calling code is finished
 * with it.
 */
char *make_method_key(JNIEnv *env, jclass clazz, jmethodID method,
                            const char *suffix)
{
	jvmdiError err;
	char *method_name = NULL;
	char *method_sig = NULL;
	char *method_suffix;
	char *result;

	err = jvmdi->GetMethodName(clazz, method, &method_name, &method_sig);
	if(err != JVMDI_ERROR_NONE) {
		report_jvmdi_error("Can't get method details", err);
		return NULL;
	}

	method_suffix = malloc(strlen(method_name) + strlen(method_sig)
	                        + strlen(suffix) + 1);

	if (method_suffix) {
		char *key_cursor = method_suffix;

		strcpy(key_cursor, method_name);
		key_cursor += strlen(method_name);

		strcpy(key_cursor, method_sig);
		key_cursor += strlen(method_sig);

		strcpy(key_cursor, suffix);
	}
else EPRINT("make_method_key() : failed to allocate mem for method_suffix");

	jvmdi->Deallocate((jbyte *)method_name);
	jvmdi->Deallocate((jbyte *)method_sig);

	if (!method_suffix) return NULL;


	result = make_class_key(env, clazz, method_suffix);

	free(method_suffix);

	return result;
}
